#!/usr/bin/env bash

set -o nounset # Treat  unset  variables  as  an error
set -o errexit # Exit immediately if a simple command (see SHELL GRAMMAR above) exits with a non-zero status.

while getopts "g" Option
do
	case ${Option} in
	g) BENCHWORK_DEBUG="YES";;
	*) echo "Usage: `basename $0` [-g]"
		exit -1;
		;;
	esac
done

# Global variables
export TOP_DIR="${PWD}/`dirname $0`"
export oldIFS=" 	
"
# Global script variables
BENCHWORK_DEBUG=${BENCHWORK_DEBUG:-"NO"}
lockfile="${TOP_DIR}/var/run_benchmark.sh.lock"

MYSQLD_ARGS=""
MYSQL_INSTALL_DB="none"
MYSQLD="none"
results_dir=""

if [ "${BENCHWORK_DEBUG}" = "YES" ]
then
	set -o xtrace
fi
if ! test -f "${lockfile}"
then
	touch "${lockfile}"
else
	echo "The lock file ${lockfile} exists. Another instance of benchwork is running?"
	exit -1
fi

# Functions definition
# Logging facility
function vlog
{
echo ""
echo "`date +"%F %T"`: `basename "$0"`: $1"
}


# Signal handlers
function wipe_processes {
vlog "Killing benchwork's processes"
killall -9 mysqld || true
killall -9 client driver || true
killall iostat vmstat sar mysqladmin mpstat top sadc || true
}

function wipe_files {
vlog "Removing temporary files"
if test -d "${TOP_DIR}/benchmarks"
then
        cd "${TOP_DIR}/benchmarks"
        for i in * sb
        do
                rm -rf "${TOP_DIR}"/var/"${i}"*
        done
fi
cd "${TOP_DIR}"
rm -rf "${TOP_DIR}"/var/mysql_server
rm -rf "${TOP_DIR}"/var/backup/*
rm -rf "${mysql_socket}"
}

function clean_on_ctrl_c {
vlog "Benchwork was interrupted by Control-C"
wipe_processes
exit
}

function clean_on_error {
vlog "Exit on error"
wipe_processes
wipe_files
exit 
}

function clean_on_exit {
vlog "Exiting"
rm -f "${lockfile}"
}

# Signal traps
trap "stop_database_server; clean_on_ctrl_c" INT
trap "clean_on_error" ERR
trap "clean_on_exit" EXIT


if test -f "${TOP_DIR}/config.sh"
then
	source "${TOP_DIR}/config.sh"
else
	echo "The configuration file ${TOP_DIR}/config.sh is not found"
	clean_on_error
fi

# Compiles MySQL sources
function prepare_database_server {

vlog "Prepare database server"
if (! test -d "`dirname "${mysql_basedir}"`/"${mysql_server}"" ) || [ "${rebuild_mysql_each_time}" = "YES" ]
then
	cd "${TOP_DIR}/src/"
	if test -d "${mysql_server}"
	then
		source "configure_${mysql_server}.sh"
	else
		echo "Can't find sources of ${mysql_server} !!!"
		echo "Please put MySQL sources to ${TOP_DIR}/src/${mysql_server}"
		echo "If you prefer precompliled binaries, install them to ${TOP_DIR}/db/${mysql_server}"
		exit_on_error
	fi
	cd "${TOP_DIR}/src/${mysql_server}" && \
	make all install && \
	cd "`dirname "${mysql_basedir}"`" && rm -f mysql && ln -s "${mysql_server}" mysql 
else
	echo "MySQL ${mysql_server} is already installed"
fi
if test -f "${mysql_defaults_path}/${mysql_defaults}"
then
	MYSQLD_ARGS="--defaults-file=${mysql_defaults_path}/${mysql_defaults} --skip-grant-tables"
fi

MYSQLD_ARGS="${MYSQLD_ARGS} --skip-grant-tables"

MYSQLD_ARGS=${MYSQLD_ARGS}" --basedir=${mysql_basedir} --datadir=${mysql_datadir} ${mysqld_args} --log-error=$results_dir/mysqld.err --socket=${mysql_socket}"
}

# Reassigns the symbolic link db/mysql to the current MySQL version
function use_database_server {

vlog "Using MySQL ${mysql_server}"
cd "`dirname "${mysql_basedir}"`" && rm -f mysql && ln -s "${mysql_server}" mysql
cd "${TOP_DIR}"
}

# Recreates database files
function prepare_database {

vlog "Prepare database"
if test -f "${mysql_basedir}/bin/mysql_install_db"
then
	MYSQL_INSTALL_DB="${mysql_basedir}/bin/mysql_install_db"
fi
if test -f "${mysql_basedir}/scripts/mysql_install_db"
then
	MYSQL_INSTALL_DB="${mysql_basedir}/scripts/mysql_install_db"
fi
if ! test -x "${MYSQL_INSTALL_DB}"
then
	echo "Can't find mysql_install_db or it is not executable"
	echo "It should be in ${mysql_basedir}/bin/ or ${mysql_basedir}/scripts/"
	clean_on_error
fi
if ! test -d "${mysql_datadir}"
then
	mkdir -p "${mysql_datadir}"
fi
if test -z "`ls "${mysql_datadir}"`"
then
	"${MYSQL_INSTALL_DB}" ${MYSQLD_ARGS}
else
	echo "${mysql_datadir} is not empty."
	read -p "Do you want to remove it? [yes|no] " answer
	if [ "$answer" = "yes" ]
	then
		rm -rf ${mysql_datadir}
		"${MYSQL_INSTALL_DB}" ${MYSQLD_ARGS}
	else
		clean_on_error
	fi
fi
}

# Checks whether MySQL is alive
function mysql_ping {

local result="0"
if test -S ${mysql_socket}
then
	result=`${MYSQL} ${MYSQL_ARGS} -e "SELECT IF(COUNT(*)>=0, 1, 1) FROM user;" -s --skip-column-names mysql 2>/dev/null` || result="0"
else
	result="0"
fi
echo $result
}

# Runs MySQL
function run_database_server {

local c=0
vlog "Running MySQL"
if test -f "${mysql_basedir}/libexec/mysqld"
then
	MYSQLD="${mysql_basedir}/libexec/mysqld"
fi
if test -f "${mysql_basedir}/bin/mysqld"
then
	MYSQLD="${mysql_basedir}/bin/mysqld"
fi
if ! test -x "${MYSQLD}"
then
	echo "mysqld is not found or isn't executable!!!"
	echo "It should be either in ${mysql_basedir}/bin/ or ${mysql_basedir}/libexec/"
	clean_on_error
fi
vlog "MySQL arguments: ${MYSQLD} ${MYSQLD_ARGS}"
if [ "`mysql_ping`" = "0" ]
then
	"${MYSQLD}" ${MYSQLD_ARGS} &
fi

while [ "`mysql_ping`" != "1" ]
do
	if [ ${c} -eq 100 ]
	then
		vlog "Can't run MySQL!"
		clean_on_error
	fi
	let c=${c}+1
	sleep 1
done
vlog "MySQL started successfully"

}

# Stops MySQL server
function stop_database_server {

local c=0
local timeout=300
vlog "Stopping MySQL server"
if [ "`mysql_ping`" = "1" ]
then
	"${MYSQLADMIN}" ${MYSQLADMIN_ARGS} shutdown || true
fi

while [ "`mysql_ping`" = "1" ]
do
	if [ ${c} -eq ${timeout} ]
	then
		vlog "MySQL is still running after ${timeout} sec! Killing it"
		killall -9 mysqld || true
	fi
	let c=${c}+1
	sleep 1
done
c=0
while ps -C mysqld > /dev/null
do
        vlog "Waiting till MySQL exits"
	if [ ${c} -eq ${timeout} ]
	then
		vlog "MySQL doesnt respond, but still running after ${timeout} sec! Killing it"
		killall -9 mysqld || true
	fi
	let c=${c}+1
        sleep 1
done

}

# Loads benchmark data into MySQL
function load_data {

vlog "Loading data"
cd "${TOP_DIR}"
source "${TOP_DIR}/benchmarks/${benchmark}/load_data.sh"
du -sb "${mysql_datadir}"  > "${results_dir}/database_size"
}

# Restores database files from the copy
function restore_data {

vlog "Restoring data from backup"
rm -rf "${mysql_datadir}"
mkdir -p "${mysql_datadir}"
cp -R "${TOP_DIR}"/var/backup/* "${mysql_datadir}"
}

function hw_details {
vlog "Gather hardware details"
test -f /var/log/dmesg && cp /var/log/dmesg "${results_dir}/hardware_details"
test -f /proc/cpuinfo && cat /proc/cpuinfo > "${results_dir}/cpuinfo"
test -f /proc/meminfo && cat /proc/meminfo > "${results_dir}/meminfo"
}

function sw_detail {
vlog "Gather software details"
uname -a >> "${results_dir}/software_details"
echo "MySQL version: ${mysql_server}" >> "${results_dir}/software_details"
}

function mysql_details {
vlog "Gather MySQL details"
echo "MySQL details" >> "${results_dir}/mysql_details"
echo "MySQL version: ${mysql_server}" >> "${results_dir}/mysql_details"
if test -x "${mysql_basedir}/bin/my_print_defaults"
then
	test -f "${mysql_defaults_path}/${mysql_defaults}" && \
	"${mysql_basedir}/bin/my_print_defaults" \
		--defaults-file="${mysql_defaults_path}/${mysql_defaults}" \
		mysqld >> "${results_dir}/mysql_details"
fi
test -f "${mysql_defaults_path}/${mysql_defaults}" && cp "${mysql_defaults_path}/${mysql_defaults}" "${results_dir}/my.cnf"
}

function run_benchmark {
vlog "Running benchmark"
source "${TOP_DIR}/benchmarks/${benchmark}/run.sh"
}

# Prepares a directory for results
# takes $1 as an iteration number
function mkdir_results {

local it="$1"
results_dir="${results_subdir}"
mkdir -p "${results_dir}/${it}"
export results_dir="${results_dir}/${it}"
}

function system_status {

local iteration_lock="${results_dir}/lock"
local pcpu=""
vlog "Collecting system statistics"
# Store CPU utilization
if [ "${collect_iostat}" = "YES" ]
then
	which iostat && (iostat -x -d ${iostat_interval} >> "${results_dir}/iostat.out" || true) &
fi
if [ "${collect_vmstat}" = "YES" ]
then
	which vmstat && (vmstat ${vmstat_interval} >> "${results_dir}/vmstat.out" || true) &
fi
if [ "${collect_mpstat}" = "YES" ]
then
	which mpstat && (mpstat -P ALL ${mpstat_interval} >> "${results_dir}/mpstat.out" || true) &
fi

if [ "${collect_sar}" = "YES" ]
then
	which sar && (sar -o "${results_dir}/sar_raw.out" ${sar_interval} 0 > /dev/null || true) &
fi

if [ "${collect_mysqladmin_ext}" = "YES" ]
then
	("${MYSQLADMIN}" ${MYSQLADMIN_ARGS} ext -i ${mysqladmin_ext_interval} -r > "${results_dir}/mysqladmin_ext.out" 2>/dev/null || true) &
fi

if [ "${collect_innodb_status}" = "YES" ]
then
	(while test -f "${iteration_lock}"
	do
        	"${MYSQL}" ${MYSQL_ARGS} -e "SHOW ENGINE INNODB STATUS\G" >> "${results_dir}/innodb_status.out" 2>/dev/null || true
		sleep ${innodb_status_interval}
	done
	)&
fi
if [ "${collect_custom_query}" = "YES" ]
then
	(while test -f "${iteration_lock}"
	do
        	"${MYSQL}" ${MYSQL_ARGS} -e "${custom_query}" >> "${results_dir}/custom_query.out" 2>/dev/null || true
		sleep ${custom_query_interval}
	done
	)&
fi
if [ "${collect_top}" = "YES" ]
then
	(while test -f "${iteration_lock}"
	do
		if [ "`uname -s`" = "FreeBSD" ]
		then
			top -b -s ${top_interval} -d 100 >> "${results_dir}/top.out"
		fi
		if [ "`uname -s`" = "Linux" ]
		then
			top -b -d ${top_interval} -n 100 >> "${results_dir}/top.out"
		fi
	done
	)&
fi


(while test -f "${iteration_lock}"
do
	echo "`date +%s`" >> "${results_dir}/iostat.out"
	echo "`date +%s`" >> "${results_dir}/vmstat.out"
	echo "`date +%s`" >> "${results_dir}/mpstat.out"
	echo "`date +%s`" >> "${results_dir}/mysqladmin_ext.out"
	sleep ${timestamp_interval}
done)&
while test -f "${iteration_lock}"
do
	if [ "${collect_ps_mysqld}" = "YES" ]
	then
		pcpu="`ps -o pcpu -C mysqld | grep -v CPU | head -1 | sed 's/ //g'`"
		if test -z "$pcpu"
		then
			pcpu="U"
		fi
		echo "`date +%s`,$pcpu" >> "${results_dir}/mysqld_cpu.csv"
	fi
	sleep ${ps_interval}
done
killall iostat || true
killall vmstat || true
killall sar || true
killall -9 mysqladmin || true
killall mpstat || true
killall top || true
killall sadc || true
}

function postrun_status {
if [ "${collect_mutex_status}" = "YES" ]
then 
	${MYSQL} ${MYSQL_ARGS} -e "SHOW MUTEX STATUS\G" >> "${results_dir}/mutex_status.out"
fi
}

function store_result {
vlog "Storing results"
source "${TOP_DIR}/benchmarks/${benchmark}/store_result.sh"
}

function process_result {
vlog "Processing results"
}

function get_n_vars {
set +u
local inv=1
local var_name
local xt
while true
do
	var_name="var${inv}"
	eval "xt=\$${var_name}" 
	if test -z "${xt}"
	then
		break
	fi
	let inv=${inv}+1
done
echo $((${inv} - 1))
set -u
}
# The function parses group variable definition and sets the value of the variables
function set_var()
{
# $1 - definition
# $2 - variable group
local definition=`echo "$1" | sed 's/[ \t]//g'`
local var_group="$2"
local i=1
local n="`echo "${definition}" | awk -F\; '{print NF}'`"
local vn=""
local k=""
local s=""
local vr=""
local vl=""

while [ "${i}" -le "${n}" ]
do
        vn=`echo "${definition}" | awk -F\; "{ print $""$i""}" | awk -F: '{ print $1}'`
        if [ "${vn}" = "$var_group" ]
        then
                k=${i}
        fi
        let i=${i}+1
done
s=`echo "${definition}" | awk -F\; "{ print $""$k""}" | awk -F: '{ print $2}' | sed 's/,/ /g'`
for i in ${s}
do
        vr=`echo ${i} | awk -F= '{ print $1}'`
        vl=`echo ${i} | cut -d= -f 2-`
        export ${vr}=${vl}
done
}


# This part generates a nested loop 
n_vars=`get_n_vars`
CORE_SH="${TOP_DIR}/var/core.sh"
rm -f "${CORE_SH}"

echo "export IFS=\",\"" >> "${CORE_SH}"
echo "# Variables which force MySQL to restart if they change" >> "${CORE_SH}"
echo "force_restart=\"mysql_server,mysqld_args,mysql_defaults_path,mysql_defaults,mysql_basedir,mysql_datadir\"" >> "${CORE_SH}"
echo "run_number=-1" >> "${CORE_SH}"

# Define undefined varX_warmup
i=1
while [ $i -le ${n_vars} ]
do
	echo "var${i}_warmup=\${var${i}_warmup:-\"NO\"}" >> "${CORE_SH}"
	let i=$i+1
done

echo "" >> "${CORE_SH}"

i=1
while [ $i -le ${n_vars} ]
do
	echo "
if [ \"\$var${i}_warmup\" = \"YES\" ]
then
	var${i}_range=\"\$var${i}_warmup_value,\$var${i}_range\"
fi" >> "${CORE_SH}"
	let i=$i+1
done
i=1
intend=""
while [ $i -le ${n_vars} ]
do
echo "${intend}v${i}_warmup=\$var${i}_warmup" >> "${CORE_SH}"
echo "${intend}for v$i in \$var${i}_range" >> "${CORE_SH}"
echo "${intend}do" >> "${CORE_SH}"
echo "${intend}	v${i}_prev=\"\${v${i}_prev:-\$v$i}\"" >> "${CORE_SH}"
echo "${intend}	for i_force_restart in \$force_restart" >> "${CORE_SH}"
echo "${intend}	do" >> "${CORE_SH}"
echo "${intend}		if [ \"\$var${i}\" = \"\$i_force_restart\" ] && [ \"xxx\$v${i}\" != \"xxx\$v${i}_prev\" ]; then stop_database_server; fi" >> "${CORE_SH}"
echo "${intend}	done" >> "${CORE_SH}"
echo "${intend}	var${i}_restart_mysqld=\${var${i}_restart_mysqld:-\"NO\"}" >> "${CORE_SH}"
echo "${intend}	if [ \"\$var${i}_restart_mysqld\" = \"YES\" ]; then stop_database_server; fi" >> "${CORE_SH}"
echo "${intend}	v${i}_prev=\"\$v$i\"" >> "${CORE_SH}"
echo "${intend}	iteration${i}=0" >> "${CORE_SH}"
echo "${intend}	var${i}_iterations=\${var${i}_iterations:-1}" >> "${CORE_SH}"
echo "${intend}	v${i}_iterations=\${var${i}_iterations}" >> "${CORE_SH}"
echo "${intend}	if [ \"\$v${i}_warmup\" = \"YES\" ] ; then v${i}_iterations=1; fi" >> "${CORE_SH}"
echo "${intend}	echo \"Warmup: \$v${i}_warmup\"" >> "${CORE_SH}"
if [ $i -eq ${n_vars} ]
then
	echo "${intend}	run_number=\$(( \${run_number} + 1))" >> "${CORE_SH}"
fi
echo "${intend}	while [ \$iteration${i} -lt \$v${i}_iterations ]" >> "${CORE_SH}"
echo "${intend}	do" >> "${CORE_SH}"
let i=$i+1
intend=$intend"		";
done

echo "${intend}export IFS=\$oldIFS" >> "${CORE_SH}"

i=1
while [ $i -le ${n_vars} ]
do
echo "
${intend}set +u
${intend}definition_name=\"\${var${i}}_definition\"
${intend}eval definition_${i}=\"\\$\$definition_name\"
${intend}set -u
${intend}if test -z \"\$definition_${i}\"
${intend}then
${intend}	export \$var${i}=\`echo \"\$v${i}\" | sed 's/ //g'\`
${intend}else
${intend}	set_var \"\$definition_${i}\" \$v${i}
${intend}	echo \"\$v${i}:(\$definition_${i})\"
${intend}fi" >> "${CORE_SH}"
let i=$i+1
done

# Here must be a part which decides whether restart MySQL

echo "
${intend}if [ \"xxx\$v${n_vars}_warmup\" = \"xxxYES\" ]
${intend}then
${intend}	mkdir_results \"\${run_number}/warmup\"
${intend}	let run_number=\${run_number}-1
${intend}else
${intend}	mkdir_results \"\${run_number}/\$iteration${n_vars}\"
${intend}fi
${intend}if [ \"\$restart_mysqld\" = \"YES\" ]
${intend}then 
${intend}	prepare_database_server
${intend}	use_database_server
${intend}	prepare_database
${intend}	load_data
${intend}else
${intend}	if [ \"\`mysql_ping\`\" = \"0\" ]
${intend}	then
${intend}		prepare_database_server
${intend}		use_database_server
${intend}		prepare_database
${intend}		load_data
${intend}		run_database_server
${intend}	fi
${intend}fi
${intend}if [ \"\`mysql_ping\`\" = \"0\" ]
${intend}then
${intend}	run_database_server
${intend}fi
${intend}echo \"Lock file: \$results_dir/lock\"
${intend}touch \$results_dir/lock
${intend}hw_details
${intend}sw_detail
${intend}mysql_details
${intend}system_status &
" >> "${CORE_SH}"
i=1 ; 
while [ $i -le ${n_vars} ]
do
	echo "${intend}echo \"var${i}: \${var${i}}=\${v${i}}\"" >> "${CORE_SH}"
	echo "${intend}echo \"var${i}: \${var${i}}=\${v${i}}\" >> \$results_dir/var.out" >> "${CORE_SH}"
	let i=$i+1
done
			
echo "
${intend}cd \${TOP_DIR}
${intend}cp config.sh \$results_dir
${intend}cp -R config.d \$results_dir
${intend}run_benchmark
${intend}rm \$results_dir/lock
${intend}postrun_status

${intend}echo \"Results are in \${results_dir}\"
${intend}if [ \"\$restart_mysqld\" = \"YES\" ]
${intend}then
${intend}	stop_database_server
${intend}	rm -rf \"\${mysql_datadir}\"/*
${intend}fi

${intend}export IFS=\",\"
" >> "${CORE_SH}"

i=1
while [ $i -le ${n_vars} ]
do
intend=`echo "${intend}" | sed 's/\t//'`
echo "${intend}	let iteration$(( ${n_vars} - $i + 1 ))=\$iteration$(( ${n_vars} - $i + 1 ))+1" >> "${CORE_SH}"
echo "${intend}	if ! test -z \$var$(( ${n_vars} - $i + 1 ))_warmup; then v$(( ${n_vars} - $i + 1 ))_warmup=\"NO\"; fi" >> "${CORE_SH}"
echo "${intend}	done" >> "${CORE_SH}"
echo "${intend}" >> "${CORE_SH}"
echo "${intend}done" >> "${CORE_SH}"
let i=$i+1
done


# MAIN part starts here


# Check if mysqld is running
mysqld_pids=`pgrep mysqld || true`
if ! test -z "${mysqld_pids}"
then
	echo "MySQL daemon is running"
	echo "Shut it down before running benchmarks"
	clean_on_error
fi

export MYSQL="${TOP_DIR}/db/mysql/bin/mysql"
export MYSQLADMIN="${TOP_DIR}/db/mysql/bin/mysqladmin"
export MYSQL_ARGS="--socket=${mysql_socket} --local-infile=1"
export MYSQLADMIN_ARGS="--socket=${mysql_socket}"

# Create the directory for results

results_metadir="${results_directory}/${results_prefix}"
if test -f "${results_metadir}/run_number"
then
	read run_number < "${results_metadir}/run_number"
	let run_number=${run_number}+1
else
	run_number=0
fi
mkdir -p "${results_metadir}/${run_number}"
echo "${run_number}" > "${results_metadir}/run_number"
results_subdir="${results_metadir}/${run_number}"

# Run generated script
source "${CORE_SH}"

if [ "${BENCHWORK_DEBUG}" = "YES" ]
then 
	rm -f "${CORE_SH}"
fi

if [ "`mysql_ping`" = "1" ]
then
	stop_database_server
fi

if [ ${keep_state} = "NO" ]
then
	wipe_files
fi
